;
; DDS generator
;
; SINE and TRIANGLE based on ...
; --------------------------
; Jesper Hansen's <jesperh@telia.com>.miniDDS ( 2000 )
; and a thread at http://www.mikrocontroller.net/topic/244609#new
; many thanks to guest Ulrich
;
; SLOW version *
; ...four times better resolution on phase 
; ...four times better resolution on frequency
; ...more than double the resolution on voltage
; ...sampling time only one extra cycle
; compared to Jesper Hansen's design
; Output frequency (using 26 BIT accumulator) :
;
;	f = deltaPhase * fClock/2^26
;
; fClock is LDS this case the CPU clock divided by the
; number of cycles to output the data ( 10 cycles )
; f = r24/r25/r26 * (16000000/10)/67108864
;
; FAST version *
; ...double the resolution on phase 
; ...double the resolution on frequency
; ...more resolution on voltage , full 8 Bit
; ...sampling time is one cycle less
; compared to Jesper Hansen's design
; Output frequency (using 25 BIT accumulator) :
;
;	f = deltaPhase * fClock/2^25
;
; fClock is LDS this case the CPU clock divided by the
; number of cycles to output the data ( 8 cycles )
; f = r24/r25/r26 * (16000000/8)/33554432
;
; SAWTOOTH
; --------
; SLOW and FAST version *
; calculated on the run, no tables used, 9 and 8 Bit
;
; PRECISE version *
; same as sine and triangle (SLOW)
;
;
; NOISE generation based on ...
; -------------------------
; a PDF from FELJC@LTAM
; (he/she does not mention his/her name)
;
; CONTROL
; -------
; The Mega8 is connected via TWI.
; Its address is 2 by default.
; Loops are interrupted by TWI activities.
; It waits for TWI START condition, reads number of register
; r24,r25,r26,r27 equals adder register
; r31 stands for mode , not stored directly to r31
; next reads value for register or mode


; Author = Werner Braun
; Target = ATmega8
; Date   = 2012-02-11
;
; PC6    = RESET
; PC5    = SCL TWI
; PC4    = SDA TWI
; PC3    = addon capacitor for filter
; PC2    = addon capacitor for filter
; PC1    = addon capacitor for filter
; PC0    = Glitch capacitor on / off (DDR)


; PB1    = D/A Data out 8
; PD0..7 = D/A Data out 0..7
;
;
.include "m8def.inc"
;
; MODE Codes
.equ SawtoothMode_precise   = 10
.equ RectangularMode_raw    = 8
.equ TriangleMode_fast      = 7
.equ TriangleMode_slow      = 6
.equ SineMode_fast          = 5
.equ SineMode_slow          = 4
.equ SawtoothMode_fast      = 3
.equ SawtoothMode_slow      = 2
.equ NoiseMode              = 1
.equ OffMode                = 0
;
;
; TWI Slave Receiver Codes
.equ TW_SR_SLA_ACK        	    = 0x60;(SLA+W received and ACK returned)													
.equ TW_SR_ARB_LOST_SLA_ACK     = 0x68;(Arbitration lost SLA+R/W as MASTER;  
                                      ; SLA+W received;  ACK returned
.equ TW_SR_GCALL_ACK      	    = 0x70;(General Call address received;ACK returned									
.equ TW_SR_ARB_LOST_GCALL_ACK   = 0x78;(Arbitration lost SLA+R/W as MASTER;
                                      ;  General Call address received;ACK returned
.equ TW_SR_DATA_ACK      	    = 0x80;(prev. adressed (SLA+W) data received;ACK returned					
.equ TW_SR_DATA_NACK      	    = 0x88;(prev. adressed (SLA+W) data received;ACK NOT returned				
.equ TW_SR_GCALL_DATA_ACK       = 0x90;(prev. adressed (General Call) data received;ACK returned			
.equ TW_SR_GCALL_DATA_NACK      = 0x98;(prev. adressed (General Call) data received;ACK NOT returned	
.equ TW_SR_STOP        		    = 0xA0;(STOP received, or repeated START condition)													

; TWI Slave Transmitter Codes
.equ TW_ST_SLA_ACK        	    = 0xA8;(SLA+R received; ACK returned)												
.equ TW_ST_ARB_LOST_SLA_ACK     = 0xB0;(Arbitration lost SLA+R/W as MASTER, ACK returned)								
.equ TW_ST_DATA_ACK      	    = 0xB8;(TWDR was tranfered;  ACK received											
.equ TW_ST_DATA_NACK      	    = 0xC0;(TWDR was tranfered;  ACK NOT received									
.equ TW_ST_LAST_DATA      	    = 0xC8;(last data Byte LDS TWDR was tranfered (TWEA = "0");Ack received		
.equ TW_NO_INFO        		    = 0xF8;(no relevant state;  TWINT = "0" no TWDR activity
.equ TW_BUS_ERROR        	    = 0x00;(Bus-Error invalid START or STOP condition; no TWDR activity				

; TWI SLAVE configuration
.equ TWI_SLAVE_ADRESSE	= 2           ; own SLAVE adress 1...127			
.equ TWI_GENERAL_CALL_enable = 0      ; 1=Generral Call enabled / 0=disabled	

; REGISTER
.def MODE      = r19
.def NUMBER    = r18
.def ADRESS	   = r17
.def TEMP	   = r16
.def DATA	   = r15
.def TEMP_SREG = r14


.cseg                              ; Start of Code-Segment
.org 0
rjmp RESET                         ; Reset Handler                       

.org TWIaddr
rjmp TWSI                          ; Two-wire Serial Interface Handler 

.org INT_VECTORS_SIZE

RESET:                        
                      
;******************************************************************************
; code
;******************************************************************************

; PORT initialization

ser TEMP
out DDRD,TEMP      ; set all PORTD bits as output

clr TEMP
out DDRB,TEMP      ; PORTB is input Bit 9 off
out PORTB,TEMP     ; internal pullup is off
ldi TEMP,0x7F      ; idle is 2.5 Volt
out PORTD,TEMP     ; preload DAC

; TWI initialization

; TWI TWBR
ldi TEMP, 65
out TWBR,TEMP
; TWI TWCR control-register
ldi TEMP, 1<<TWINT|1<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|1<<TWEN|1<<TWIE
out TWCR,TEMP
; TWI TWAR (only SLAVE) adress-register		
ldi TEMP,(TWI_SLAVE_ADRESSE<<TWA0 | TWI_GENERAL_CALL_ENABLE<<TWGCE)
out TWAR,TEMP


; start values			
clr DATA
clr ADRESS
clr MODE

;                   ***** DEBUG ********
;ldi MODE,SawtoothMode_precise
;ldi MODE,SineMode_fast
;ldi MODE,SawtoothMode_fast

; setup adder registers		
clr r24
clr r25	      
clr r26
clr r27


		
Idle:

; clear up STACK
ldi	TEMP,0xFF
out	SPL,TEMP		
clr Temp
out SPH,TEMP

; global enable interrupts
sei				  

; new game
cpi MODE,OffMode  
; wait for a job
breq Idle

cpi MODE,NoiseMode
brne next1
rjmp NoiseLoop
next1:

cpi MODE,SineMode_slow
brne next2
ldi r31,high(2*SineTable)
rjmp TableLoop
next2:

cpi MODE,SineMode_fast
brne next3
; point to long sine table
ldi r31,high(2*SineTable)
ldi r30,low(2*SineTable)
; point to SRAM starting  $3FF +1
; copy every second Byte to SRAM
ldi YH,0x04
ldi YL,0x00
sine_to_255:
lpm r16,z+
ror r16
andi r16,0x7F
st -y,r16
lpm r16,z+
cpi YL,0x00
brne sine_to_255
ldi YH,0x03
ldi YL,0x00
sine_to_510:
lpm r16,z+
ror r16
ori r16,0x80
st -y,r16
lpm r16,z+
cpi YL,0x00
brne sine_to_510
ldi YH,0x01
ldi YL,0x00
ldi r31,0x03
ldi r30,0x00
sine_to_768:
ld  r16,z+
st  y+,r16
cpi r31,0x04
brlo sine_to_768
rjmp TableLoop_fast
next3:

cpi MODE,SawtoothMode_fast
brne next4
rjmp ST_fast
next4:

cpi MODE,SawtoothMode_slow
brne next5
rjmp ST_slow
next5:

cpi MODE,TriangleMode_slow
brne next6
ldi r31,high(2*TriangleTable)
rjmp TableLoop
next6:

cpi MODE,TriangleMode_fast
brne next7
; point to long Triangle table
ldi r31,high(2*TriangleTable)
ldi r30,low(2*TriangleTable)
; point to SRAM starting  $3FF +1
; copy every second Byte to SRAM
ldi YH,0x04
ldi YL,0x00
Triangle_to_255:
lpm r16,z+
ror r16
andi r16,0x7F
st -y,r16
lpm r16,z+
cpi YL,0x00
brne Triangle_to_255
ldi YH,0x03
ldi YL,0x00
Triangle_to_510:
lpm r16,z+
ror r16
ori r16,0x80
st -y,r16
lpm r16,z+
cpi YL,0x00
brne Triangle_to_510
ldi YH,0x01
ldi YL,0x00
ldi r31,0x03
ldi r30,0x00
Triangle_to_768:
ld  r16,z+
st  y+,r16
cpi r31,0x04
brlo Triangle_to_768
rjmp TableLoop_fast
next7:

cpi MODE,RectangularMode_raw
brne next8
rjmp RectangularMode
next8:

cpi MODE,SawtoothMode_precise
brne next9
ldi r31,high(2*SawtoothTable)
ldi r30,0xFF
out PORTD,r30
rjmp TableLoop

next9:
rjmp Idle           ; NO valid MODE !



TableLoop_fast:		

;   2 Mc sampling rate , 360/512 , 8 Bit

;	r28,r29,r30,r31 is the phase accumulator
;  	r24,r25,r26,r27 is the adder value determining frequency
;
; 	add value to accumulator
;	load Byte from current table in SRAM
;	output Byte to port
;	repeat 

	in   TEMP,DDRB ; Bit 8 is off
	cbr  Temp,2
	out  DDRB,TEMP
	in   TEMP,PORTB
	cbr  TEMP,2
	out  PORTB,TEMP
	in   TEMP,DDRC ; deglitch not needed
	cbr  TEMP,1
	out  DDRC,TEMP
    in   TEMP,PORTC
	cbr  TEMP,1
	out  PORTC,TEMP

	ldi  TEMP,0xFF  ; use all Bits for DAC
	out  DDRD,TEMP

	ldi  r27,0    
	ldi  r31,3

    TL:             ; cycl.
	ld   r0,z       ; 2  
	out  PORTD,r0   ; 1  update BIT 0..7
	sub  r28,r24    ; 1
	sbc  r29,r25    ; 1
	sbc  r30,r26    ; 1
	sbc  r31,r27    ; 1
	ori  r31,0x02  	; 1  limit to 1.5 * table length
	ld   r0,z       ; 2
	out  PORTD,r0   ; 1  update BIT 0..7
	sub  r28,r24    ; 1
	sbc  r29,r25    ; 1
	sbc  r30,r26    ; 1
	sbc  r31,r27    ; 1   
	ld   r0,z       ; 2  
	sub  r28,r24    ; 1
	out  PORTD,r0   ; 1  update BIT 0..7
	sbc  r29,r25    ; 1
	sbc  r30,r26    ; 1
	sbc  r31,r27    ; 1
	ori  r31,0x02	; 1  limit to 1.5 * table length
	ld   r0,z       ; 2
	sub  r28,r24    ; 1
	out  PORTD,r0   ; 1  update BIT 0..7
	sbc  r29,r25    ; 1
	sbc  r30,r26    ; 1
	sbc  r31,r27    ; 1
	rjmp TL         ; 2 => 32 / 4 = 8 cycles


TableLoop:		

;   1.6 Mc sampling rate , 360/1024 , 9 Bit

;	r28,r29,r30,r31 is the phase accumulator
;  	r24,r25,r26,r27 is the adder value determining frequency
;
; 	add value to accumulator
;	load Byte from current table in FLASH
;	output Byte to port
;	repeat 

	in   TEMP,DDRB  ; Bit 8 is on
	sbr  TEMP,2
	out  DDRB,TEMP
    in   TEMP,PORTB
	cbr  TEMP,2
	out  PORTB,TEMP
	in   TEMP,DDRC  ; deglitch is needed
	ori  TEMP,1
	out  DDRC,TEMP
    in   TEMP,PORTC
	cbr  TEMP,1
	out  PORTC,TEMP

	ldi  TEMP,0xFF  ; use all Bits for DAC
	out  DDRD,TEMP

	clr r27

    TL1:            ; cycl.
	lpm  r0,z       ; 3  
	out  PORTD,r0   ; 1  update BIT 0..7
	out  PortB,r31  ; 1  update Bit 8
	add  r28,r24    ; 1
	adc  r29,r25    ; 1
	adc  r30,r26    ; 1
	adc  r31,r27    ; 1
	cbr  r31,4  	; 1  limit to 1.5 * table length
	lpm  r0,z       ; 3
	out  PORTD,r0   ; 1  update BIT 0..7
	out  PortB,r31  ; 1  update Bit 8
	add  r28,r24    ; 1
	adc  r29,r25    ; 1
	adc  r30,r26    ; 1
	adc  r31,r27    ; 1   
	lpm  r0,z       ; 3  
	add  r28,r24    ; 1
	out  PORTD,r0   ; 1  update BIT 0..7
	out  PortB,r31  ; 1  update Bit 8
	adc  r29,r25    ; 1
	adc  r30,r26    ; 1
	adc  r31,r27    ; 1
	cbr  r31,4	    ; 1  limit to 1.5 * table length
	lpm  r0,z       ; 3
	add  r28,r24    ; 1
	out  PORTD,r0   ; 1  update BIT 0..7
	out  PortB,r31  ; 1  update Bit 8
	adc  r29,r25    ; 1
	adc  r30,r26    ; 1
	adc  r31,r27    ; 1
	rjmp TL1        ; 2 => 40 / 4 = 10 cycles

SawtoothLoop:

    ST_slow:

    ; 2 Mc sampling rate , 360/512 , 9 Bit

	in   TEMP,DDRB  ; Bit 8 is on
	sbr  TEMP,2
	out  DDRB,TEMP
    in   TEMP,PORTB
	cbr  TEMP,2
	out  PORTB,TEMP
	in   TEMP,DDRC  ; deglitch is needed
	ori  TEMP,1
	out  DDRC,TEMP
    in  TEMP,PORTC
	cbr  TEMP,1
	out  PORTC,TEMP

	ldi  TEMP,0xFF   ; use all Bits for DAC
	out  DDRD,TEMP

	ldi  r31,1
	ldi  r30,0xFF    ; start half way
	out  PORTD,r30
	add  r28,r24 
	    
    STL1:
	adc  r29,r25     ; 1
	adc  r30,r26     ; 1
    adc  r31,r27     ; 1  
	ori  r31,1       ; 1
	out  PORTD,r30   ; 1  update Bit 0..7
	out  PORTB,r31   ; 1  update Bit 8
	add  r28,r24     ; 1
	adc  r29,r25     ; 1
	adc  r30,r26     ; 1
    adc  r31,r27     ; 1  
	add  r28,r24     ; 1
	ori  r31,1       ; 1
	out  PORTD,r30   ; 1  update Bit 0..7
	out  PORTB,r31   ; 1  update Bit 8
    rjmp STL1;       ; 2  16 / 2 = 8 cycles

    ST_fast:

    ; 3,2 Mc sampling rate , 360/256 , 8 Bit

	in   TEMP,DDRB   ; Bit 8 is off
	cbr  Temp,2
	out  DDRB,TEMP
	in   TEMP,PORTB
	cbr  TEMP,2
	out  PORTB,TEMP
	in   TEMP,DDRC   ; deglitch not needed
	cbr  TEMP,1
	out  DDRC,TEMP
    in   TEMP,PORTC
	cbr  TEMP,1
	out  PORTC,TEMP

	ldi  TEMP,0xFF    ; use all Bits for DAC
	out  DDRD,TEMP

	add  r28,r24
	ldi  r30,0x7F    ; start half way
	out  PORTD,r30    
	 
    STL2:
	adc  r29,r25     ; 1
	adc  r30,r26     ; 1
	out  PORTD,r30   ; 1  update Bit 0..7
	add  r28,r24     ; 1
	adc  r29,r25     ; 1
	adc  r30,r26     ; 1
	add  r28,r24     ; 1
	out  PORTD,r30   ; 1  update Bit 0..7
    rjmp STL2;       ; 2  10 / 2 = 5 cycles


RectangularMode:

    ; 1,77 Mc sampling rate , 360/256 , 8 Bit

	in   TEMP,DDRB   ; Bit 8 is off
	cbr  Temp,2
	out  DDRB,TEMP
	in   TEMP,PORTB
	cbr  TEMP,2
	out  PORTB,TEMP
	in   TEMP,DDRC   ; deglitch not needed
	cbr  TEMP,1
	out  DDRC,TEMP
    in   TEMP,PORTC
	cbr  TEMP,1
	out  PORTC,TEMP

	ldi  TEMP,0xFF   ; 
	out  DDRD,TEMP
    
	ldi r31,0xFF

	add  r28,r24     
	RM1: 
	adc  r29,r25     ; 1
	adc  r30,r26     ; 1
    brcc out1        ; 2
	com  r31
	out1:
	out  PORTD,r31   ; 1  update Bit 0..7
	add  r28,r24     ; 1
	adc  r29,r25     ; 1
	adc  r30,r26     ; 1
    brcc out2        ; 2
	com  r31
	out2:
	add  r28,r24     ; 1
	out  PORTD,r31   ; 1  update Bit 0..7
    rjmp RM1;        ; 2  14 / 2 = 7 cycles

NoiseLoop:

;   Linear Feedback Register LFR

    in  TEMP,DDRB
    ori TEMP,2
    out DDRB,TEMP        ; Bit 8,9 = PORTB0..1
    in  TEMP,DDRC        ; deglitch not needed
    cbr TEMP,1
    out DDRC,TEMP
    in  TEMP,PORTC
    cbr TEMP,1
    out PORTC,TEMP

	ldi  TEMP,0xFF        ; use all Bits for DAC
	out  DDRD,TEMP

    ldi r21,0            ; used for XOR
    ldi r22,0            ; used for XOR
    ldi r28,0x5C         ; starting value
    ldi r29,0xD3
    ldi r30,0xBC
    ldi r31,0xB4
  N0:
    ldi R20,32           ; 4*8 = 32 BIT
  N1:
    rol r28              ; rotate 3 bytes over carry
    rol r30
    rol r31
    bst r31,7     
    bld r21,0   
    bst r30,5    
    bld r22,0  
    eor r21,r22   
    bst r28,2     
    bld r22,0    
    eor r21,r22    
    bst r28,0    
    bld r22,0     
    eor r21,r22     
    bst r21,0   
    bld r28,0  
    out PortB,r31        ; have some glitch
    out PortD,r30
    dec r20              ; finished ?
    brne N1              ; no, go on
    rjmp N0


; TWI INTERRUPPT-Routine		

TWSI:

	in TEMP_SREG,SREG		            ; save SREG 
	
	in  TEMP, TWSR			            ; get TWI status				
	andi TEMP, 0xF8			            ; mask Bit0...2 (prescaler und reserve-BIT)
	cpi TEMP, TW_SR_SLA_ACK			    ; x60	
	breq TWSI_TW_SR_SLA_ACK
	cpi TEMP, TW_SR_ARB_LOST_SLA_ACK 	; x68	
	breq TWSI_END
	cpi TEMP, TW_SR_GCALL_ACK		    ; x70	
	breq TWSI_END  
	cpi TEMP, TW_SR_ARB_LOST_GCALL_ACK	; x78	
	breq TWSI_END
	cpi TEMP, TW_SR_DATA_ACK      		; x80	
	breq TWSI_TW_SR_DATA_ACK      	
	cpi TEMP, TW_SR_DATA_NACK      		; x88	
	breq TWSI_END
	cpi TEMP, TW_SR_GCALL_DATA_ACK   	; x90	
	breq TWSI_END
	cpi TEMP, TW_SR_GCALL_DATA_NACK		; x98	
	breq TWSI_END
	cpi TEMP, TW_SR_STOP        		; xA0	
	breq TWSI_END       	
; 	
	cpi TEMP, TW_ST_SLA_ACK        		; 0xA8 	
	breq TWSI_TW_ST_SLA_ACK	
	cpi TEMP, TW_ST_ARB_LOST_SLA_ACK	; 0xB0 	
	breq TWSI_END
	cpi TEMP, TW_ST_DATA_ACK      		; 0xB8	
	breq TWSI_TW_ST_DATA_ACK
	cpi TEMP, TW_ST_DATA_NACK      		; 0xC0	
	breq TWSI_TW_ST_DATA_NACK
	cpi TEMP, TW_ST_LAST_DATA      		; 0xC8	
	breq TWSI_END 
	cpi TEMP, TW_NO_INFO        		; 0xF8	
	breq TWSI_END
	rcall TWI_ERROR        			    ; 0x00	

TWSI_END:	

	; clear TWINT-Flag 	
	ldi TEMP, 1<<TWINT|1<<TWEA|1<<TWEN|1<<TWIE
	out TWCR,TEMP
	; restore SREG 
	out SREG,TEMP_SREG		      
    
    reti


;	SLAVE TRANSMITTER		
    TWSI_TW_ST_SLA_ACK:		; xA8 	10101000 (SLA+R received; ACK returned)		
	out TWDR, DATA			; DATA
	rjmp TWSI_END
    TWSI_TW_ST_DATA_ACK:	; xB8	10111000 (TWDR was tranfered;  ACK received
	out TWDR, DATA			; DATA
	rjmp TWSI_END
    TWSI_TW_ST_DATA_NACK:	; xC0	11000000 (TWDR was tranfered;  ACK NOT received
	out TWDR, DATA			; DATA
	rjmp TWSI_END

;   SLAVE RECEIVER				
    TWSI_TW_SR_SLA_ACK:		; x60	01100000 (SLA+W received-ACK returned)
	clr NUMBER			    ; START next is register number to come in
	rjmp TWSI_END
    TWSI_TW_SR_DATA_ACK:	; x80	10000000 (prev. adressed (SLA+W) data received;  ACK returned	
	cpi NUMBER, 0
	brne VALUE
	in ADRESS, TWDR
	inc NUMBER
 	rjmp TWSI_END
    VALUE:
	in DATA,TWDR			; get DATA
	cpi ADRESS,0x18			; r24 ?
    brne l25
	mov r24,DATA			; yes, set
	rjmp TWSI_END
    l25:
 	cpi ADRESS,0x19			; r25 ?
	brne l26
	mov r25,DATA			; yes, set
	rjmp TWSI_END
    l26:
	cpi ADRESS,0x1a			; r26 ?
	brne l27
	mov r26,DATA			; yes, set
	rjmp TWSI_END
    l27:
	cpi ADRESS,0x1c			; r27 ?
	brne l29
	mov r28,DATA			; yes, set
	rjmp TWSI_END
    l28:
	cpi ADRESS,0x1c			; r28 ?
	brne l29
	mov r28,DATA			; yes, set
	rjmp TWSI_END
    l29:
	cpi ADRESS,0x1d			; r29 ?
	brne l30
	mov r29,DATA			; yes, set
	rjmp TWSI_END
    l30:
 	cpi ADRESS,0x1e			; r30 ?
	brne l31
	mov r30,DATA			; yes, set
	rjmp TWSI_END
    l31:
	cpi ADRESS,0x1f			; r31 ?
	brne TWSI_END
	mov MODE,DATA			; yes, set
    ; clear TWINT-Flag 	
    ldi TEMP, 1<<TWINT|1<<TWEA|1<<TWEN|1<<TWIE
    out TWCR,TEMP
	rjmp Idle


TWI_ERROR:

;   TWI STOP				
    ldi TEMP, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)
    out TWCR, TEMP
;   TWI reinit				
    ldi TEMP, (0<<TWEN)
    out TWCR, TEMP
    ldi TEMP, 65
    out TWBR,TEMP
    ; TWI TWCR control-register
    ldi TEMP, 1<<TWINT|1<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|1<<TWEN|1<<TWIE
    out TWCR,TEMP
    ; TWI TWAR (SLAVE only) adress-register		
    ldi TEMP,(TWI_SLAVE_ADRESSE<<TWA0 | TWI_GENERAL_CALL_ENABLE<<TWGCE)
    out TWAR,TEMP
ret


;******************************************************************************
; data tables
;******************************************************************************

; force table to begin at 256 Byte boundary

.org 0x400

SawtoothTable:

.db 0,0,1,1,2,2,3,3,4,4,5,5,6,6,7,7
.db 8,8,9,9,10,10,11,11,12,12,13,13,14,14,15,15
.db 16,16,17,17,18,18,19,19,20,20,21,21,22,22,23,23
.db 24,24,25,25,26,26,27,27,28,28,29,29,30,30,31,31
.db 32,32,33,33,34,34,35,35,36,36,37,37,38,38,39,39
.db 40,40,41,41,42,42,43,43,44,44,45,45,46,46,47,47
.db 48,48,49,49,50,50,51,51,52,52,53,53,54,54,55,55
.db 56,56,57,57,58,58,59,59,60,60,61,61,62,62,63,63
.db 64,64,65,65,66,66,67,67,68,68,69,69,70,70,71,71
.db 72,72,73,73,74,74,75,75,76,76,77,77,78,78,79,79
.db 80,80,81,81,82,82,83,83,84,84,85,85,86,86,87,87
.db 88,88,89,89,90,90,91,91,92,92,93,93,94,94,95,95
.db 96,96,97,97,98,98,99,99,100,100,101,101,102,102,103,103
.db 104,104,105,105,106,106,107,107,108,108,109,109,110,110,111,111
.db 112,112,113,113,114,114,115,115,116,116,117,117,118,118,119,119
.db 120,120,121,121,122,122,123,123,124,124,125,125,126,126,127,127
.db 128,128,129,129,130,130,131,131,132,132,133,133,134,134,135,135
.db 136,136,137,137,138,138,139,139,140,140,141,141,142,142,143,143
.db 144,144,145,145,146,146,147,147,148,148,149,149,150,150,151,151
.db 152,152,153,153,154,154,155,155,156,156,157,157,158,158,159,159
.db 160,160,161,161,162,162,163,163,164,164,165,165,166,166,167,167
.db 168,168,169,169,170,170,171,171,172,172,173,173,174,174,175,175
.db 176,176,177,177,178,178,179,179,180,180,181,181,182,182,183,183
.db 184,184,185,185,186,186,187,187,188,188,189,189,190,190,191,191
.db 192,192,193,193,194,194,195,195,196,196,197,197,198,198,199,199
.db 200,200,201,201,202,202,203,203,204,204,205,205,206,206,207,207
.db 208,208,209,209,210,210,211,211,212,212,213,213,214,214,215,215
.db 216,216,217,217,218,218,219,219,220,220,221,221,222,222,223,223
.db 224,224,225,225,226,226,227,227,228,228,229,229,230,230,231,231
.db 232,232,233,233,234,234,235,235,236,236,237,237,238,238,239,239
.db 240,240,241,241,242,242,243,243,244,244,245,245,246,246,247,247
.db 248,248,249,249,250,250,251,251,252,252,253,253,254,254,255,255

.db 0,0,1,1,2,2,3,3,4,4,5,5,6,6,7,7
.db 8,8,9,9,10,10,11,11,12,12,13,13,14,14,15,15
.db 16,16,17,17,18,18,19,19,20,20,21,21,22,22,23,23
.db 24,24,25,25,26,26,27,27,28,28,29,29,30,30,31,31
.db 32,32,33,33,34,34,35,35,36,36,37,37,38,38,39,39
.db 40,40,41,41,42,42,43,43,44,44,45,45,46,46,47,47
.db 48,48,49,49,50,50,51,51,52,52,53,53,54,54,55,55
.db 56,56,57,57,58,58,59,59,60,60,61,61,62,62,63,63
.db 64,64,65,65,66,66,67,67,68,68,69,69,70,70,71,71
.db 72,72,73,73,74,74,75,75,76,76,77,77,78,78,79,79
.db 80,80,81,81,82,82,83,83,84,84,85,85,86,86,87,87
.db 88,88,89,89,90,90,91,91,92,92,93,93,94,94,95,95
.db 96,96,97,97,98,98,99,99,100,100,101,101,102,102,103,103
.db 104,104,105,105,106,106,107,107,108,108,109,109,110,110,111,111
.db 112,112,113,113,114,114,115,115,116,116,117,117,118,118,119,119
.db 120,120,121,121,122,122,123,123,124,124,125,125,126,126,127,127
.db 128,128,129,129,130,130,131,131,132,132,133,133,134,134,135,135
.db 136,136,137,137,138,138,139,139,140,140,141,141,142,142,143,143
.db 144,144,145,145,146,146,147,147,148,148,149,149,150,150,151,151
.db 152,152,153,153,154,154,155,155,156,156,157,157,158,158,159,159
.db 160,160,161,161,162,162,163,163,164,164,165,165,166,166,167,167
.db 168,168,169,169,170,170,171,171,172,172,173,173,174,174,175,175
.db 176,176,177,177,178,178,179,179,180,180,181,181,182,182,183,183
.db 184,184,185,185,186,186,187,187,188,188,189,189,190,190,191,191
.db 192,192,193,193,194,194,195,195,196,196,197,197,198,198,199,199
.db 200,200,201,201,202,202,203,203,204,204,205,205,206,206,207,207
.db 208,208,209,209,210,210,211,211,212,212,213,213,214,214,215,215
.db 216,216,217,217,218,218,219,219,220,220,221,221,222,222,223,223
.db 224,224,225,225,226,226,227,227,228,228,229,229,230,230,231,231
.db 232,232,233,233,234,234,235,235,236,236,237,237,238,238,239,239
.db 240,240,241,241,242,242,243,243,244,244,245,245,246,246,247,247
.db 248,248,249,249,250,250,251,251,252,252,253,253,254,254,255,255

.db 0,0,1,1,2,2,3,3,4,4,5,5,6,6,7,7
.db 8,8,9,9,10,10,11,11,12,12,13,13,14,14,15,15
.db 16,16,17,17,18,18,19,19,20,20,21,21,22,22,23,23
.db 24,24,25,25,26,26,27,27,28,28,29,29,30,30,31,31
.db 32,32,33,33,34,34,35,35,36,36,37,37,38,38,39,39
.db 40,40,41,41,42,42,43,43,44,44,45,45,46,46,47,47
.db 48,48,49,49,50,50,51,51,52,52,53,53,54,54,55,55
.db 56,56,57,57,58,58,59,59,60,60,61,61,62,62,63,63
.db 64,64,65,65,66,66,67,67,68,68,69,69,70,70,71,71
.db 72,72,73,73,74,74,75,75,76,76,77,77,78,78,79,79
.db 80,80,81,81,82,82,83,83,84,84,85,85,86,86,87,87
.db 88,88,89,89,90,90,91,91,92,92,93,93,94,94,95,95
.db 96,96,97,97,98,98,99,99,100,100,101,101,102,102,103,103
.db 104,104,105,105,106,106,107,107,108,108,109,109,110,110,111,111
.db 112,112,113,113,114,114,115,115,116,116,117,117,118,118,119,119
.db 120,120,121,121,122,122,123,123,124,124,125,125,126,126,127,127
.db 128,128,129,129,130,130,131,131,132,132,133,133,134,134,135,135
.db 136,136,137,137,138,138,139,139,140,140,141,141,142,142,143,143
.db 144,144,145,145,146,146,147,147,148,148,149,149,150,150,151,151
.db 152,152,153,153,154,154,155,155,156,156,157,157,158,158,159,159
.db 160,160,161,161,162,162,163,163,164,164,165,165,166,166,167,167
.db 168,168,169,169,170,170,171,171,172,172,173,173,174,174,175,175
.db 176,176,177,177,178,178,179,179,180,180,181,181,182,182,183,183
.db 184,184,185,185,186,186,187,187,188,188,189,189,190,190,191,191
.db 192,192,193,193,194,194,195,195,196,196,197,197,198,198,199,199
.db 200,200,201,201,202,202,203,203,204,204,205,205,206,206,207,207
.db 208,208,209,209,210,210,211,211,212,212,213,213,214,214,215,215
.db 216,216,217,217,218,218,219,219,220,220,221,221,222,222,223,223
.db 224,224,225,225,226,226,227,227,228,228,229,229,230,230,231,231
.db 232,232,233,233,234,234,235,235,236,236,237,237,238,238,239,239
.db 240,240,241,241,242,242,243,243,244,244,245,245,246,246,247,247
.db 248,248,249,249,250,250,251,251,252,252,253,253,254,254,255,255

.org 0x800

SineTable:
		
.db 255,254,252,251,249,248,246,245,243,241,240,238,237,235,234,232
.db 230,229,227,226,224,223,221,220,218,216,215,213,212,210,209,207
.db 206,204,203,201,200,198,196,195,193,192,190,189,187,186,184,183
.db 181,180,178,177,175,174,172,171,169,168,166,165,164,162,161,159
.db 158,156,155,153,152,151,149,148,146,145,143,142,141,139,138,136
.db 135,134,132,131,130,128,127,125,124,123,121,120,119,117,116,115
.db 114,112,111,110,108,107,106,105,103,102,101,100,98,97,96,95
.db 93,92,91,90,89,87,86,85,84,83,82,80,79,78,77,76
.db 75,74,73,72,70,69,68,67,66,65,64,63,62,61,60,59
.db 58,57,56,55,54,53,52,51,50,49,48,48,47,46,45,44
.db 43,42,41,40,40,39,38,37,36,36,35,34,33,32,32,31
.db 30,29,29,28,27,27,26,25,25,24,23,23,22,21,21,20
.db 19,19,18,18,17,17,16,15,15,14,14,13,13,12,12,11
.db 11,11,10,10,9,9,8,8,8,7,7,7,6,6,6,5
.db 5,5,4,4,4,3,3,3,3,3,2,2,2,2,2,1
.db 1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0
.db 0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1
.db 1,1,2,2,2,2,2,3,3,3,3,3,4,4,4,5
.db 5,5,6,6,6,7,7,7,8,8,8,9,9,10,10,11
.db 11,11,12,12,13,13,14,14,15,15,16,17,17,18,18,19
.db 19,20,21,21,22,23,23,24,25,25,26,27,27,28,29,29
.db 30,31,32,32,33,34,35,36,36,37,38,39,40,40,41,42
.db 43,44,45,46,47,48,48,49,50,51,52,53,54,55,56,57
.db 58,59,60,61,62,63,64,65,66,67,68,69,70,72,73,74
.db 75,76,77,78,79,80,82,83,84,85,86,87,89,90,91,92
.db 93,95,96,97,98,100,101,102,103,105,106,107,108,110,111,112
.db 114,115,116,117,119,120,121,123,124,125,127,128,130,131,132,134
.db 135,136,138,139,141,142,143,145,146,148,149,151,152,153,155,156
.db 158,159,161,162,164,165,166,168,169,171,172,174,175,177,178,180
.db 181,183,184,186,187,189,190,192,193,195,196,198,200,201,203,204
.db 206,207,209,210,212,213,215,216,218,220,221,223,224,226,227,229
.db 230,232,234,235,237,238,240,241,243,245,246,248,249,251,252,254

.db 0,1,3,4,6,7,9,10,12,14,15,17,18,20,21,23
.db 25,26,28,29,31,32,34,35,37,39,40,42,43,45,46,48
.db 49,51,52,54,55,57,59,60,62,63,65,66,68,69,71,72
.db 74,75,77,78,80,81,83,84,86,87,89,90,91,93,94,96
.db 97,99,100,102,103,104,106,107,109,110,112,113,114,116,117,119
.db 120,121,123,124,125,127,128,130,131,132,134,135,136,138,139,140
.db 141,143,144,145,147,148,149,150,152,153,154,155,157,158,159,160
.db 162,163,164,165,166,168,169,170,171,172,173,175,176,177,178,179
.db 180,181,182,183,185,186,187,188,189,190,191,192,193,194,195,196
.db 197,198,199,200,201,202,203,204,205,206,207,207,208,209,210,211
.db 212,213,214,215,215,216,217,218,219,219,220,221,222,223,223,224
.db 225,226,226,227,228,228,229,230,230,231,232,232,233,234,234,235
.db 236,236,237,237,238,238,239,240,240,241,241,242,242,243,243,244
.db 244,244,245,245,246,246,247,247,247,248,248,248,249,249,249,250
.db 250,250,251,251,251,252,252,252,252,252,253,253,253,253,253,254
.db 254,254,254,254,254,254,255,255,255,255,255,255,255,255,255,255
.db 255,255,255,255,255,255,255,255,255,255,255,254,254,254,254,254
.db 254,254,253,253,253,253,253,252,252,252,252,252,251,251,251,250
.db 250,250,249,249,249,248,248,248,247,247,247,246,246,245,245,244
.db 244,244,243,243,242,242,241,241,240,240,239,238,238,237,237,236
.db 236,235,234,234,233,232,232,231,230,230,229,228,228,227,226,226
.db 225,224,223,223,222,221,220,219,219,218,217,216,215,215,214,213
.db 212,211,210,209,208,207,207,206,205,204,203,202,201,200,199,198
.db 197,196,195,194,193,192,191,190,189,188,187,186,185,183,182,181
.db 180,179,178,177,176,175,173,172,171,170,169,168,166,165,164,163
.db 162,160,159,158,157,155,154,153,152,150,149,148,147,145,144,143
.db 141,140,139,138,136,135,134,132,131,130,128,127,125,124,123,121
.db 120,119,117,116,114,113,112,110,109,107,106,104,103,102,100,99
.db 97,96,94,93,91,90,89,87,86,84,83,81,80,78,77,75
.db 74,72,71,69,68,66,65,63,62,60,59,57,55,54,52,51
.db 49,48,46,45,43,42,40,39,37,35,34,32,31,29,28,26
.db 25,23,21,20,18,17,15,14,12,10,9,7,6,4,3,1

.db 255,254,252,251,249,248,246,245,243,241,240,238,237,235,234,232
.db 230,229,227,226,224,223,221,220,218,216,215,213,212,210,209,207
.db 206,204,203,201,200,198,196,195,193,192,190,189,187,186,184,183
.db 181,180,178,177,175,174,172,171,169,168,166,165,164,162,161,159
.db 158,156,155,153,152,151,149,148,146,145,143,142,141,139,138,136
.db 135,134,132,131,130,128,127,125,124,123,121,120,119,117,116,115
.db 114,112,111,110,108,107,106,105,103,102,101,100,98,97,96,95
.db 93,92,91,90,89,87,86,85,84,83,82,80,79,78,77,76
.db 75,74,73,72,70,69,68,67,66,65,64,63,62,61,60,59
.db 58,57,56,55,54,53,52,51,50,49,48,48,47,46,45,44
.db 43,42,41,40,40,39,38,37,36,36,35,34,33,32,32,31
.db 30,29,29,28,27,27,26,25,25,24,23,23,22,21,21,20
.db 19,19,18,18,17,17,16,15,15,14,14,13,13,12,12,11
.db 11,11,10,10,9,9,8,8,8,7,7,7,6,6,6,5
.db 5,5,4,4,4,3,3,3,3,3,2,2,2,2,2,1
.db 1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0
.db 0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1
.db 1,1,2,2,2,2,2,3,3,3,3,3,4,4,4,5
.db 5,5,6,6,6,7,7,7,8,8,8,9,9,10,10,11
.db 11,11,12,12,13,13,14,14,15,15,16,17,17,18,18,19
.db 19,20,21,21,22,23,23,24,25,25,26,27,27,28,29,29
.db 30,31,32,32,33,34,35,36,36,37,38,39,40,40,41,42
.db 43,44,45,46,47,48,48,49,50,51,52,53,54,55,56,57
.db 58,59,60,61,62,63,64,65,66,67,68,69,70,72,73,74
.db 75,76,77,78,79,80,82,83,84,85,86,87,89,90,91,92
.db 93,95,96,97,98,100,101,102,103,105,106,107,108,110,111,112
.db 114,115,116,117,119,120,121,123,124,125,127,128,130,131,132,134
.db 135,136,138,139,141,142,143,145,146,148,149,151,152,153,155,156
.db 158,159,161,162,164,165,166,168,169,171,172,174,175,177,178,180
.db 181,183,184,186,187,189,190,192,193,195,196,198,200,201,203,204
.db 206,207,209,210,212,213,215,216,218,220,221,223,224,226,227,229
.db 230,232,234,235,237,238,240,241,243,245,246,248,249,251,252,254


.org 0xC00

TriangleTable:

.db 255,254,253,252,251,250,249,248,247,246,245,244,243,242,241,240
.db 239,238,237,236,235,234,233,232,231,230,229,228,227,226,225,224
.db 223,222,221,220,219,218,217,216,215,214,213,212,211,210,209,208
.db 207,206,205,204,203,202,201,200,199,198,197,196,195,194,193,192
.db 191,190,189,188,187,186,185,184,183,182,181,180,179,178,177,176
.db 175,174,173,172,171,170,169,168,167,166,165,164,163,162,161,160
.db 159,158,157,156,155,154,153,152,151,150,149,148,147,146,145,144
.db 143,142,141,140,139,138,137,136,135,134,133,132,131,130,129,128
.db 127,126,125,124,123,122,121,120,119,118,117,116,115,114,113,112
.db 111,110,109,108,107,106,105,104,103,102,101,100,99,98,97,96
.db 95,94,93,92,91,90,89,88,87,86,85,84,83,82,81,80
.db 79,78,77,76,75,74,73,72,71,70,69,68,67,66,65,64
.db 63,62,61,60,59,58,57,56,55,54,53,52,51,50,49,48
.db 47,46,45,44,43,42,41,40,39,38,37,36,35,34,33,32
.db 31,30,29,28,27,26,25,24,23,22,21,20,19,18,17,16
.db 15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0
.db 0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15
.db 16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31
.db 32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47
.db 48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63
.db 64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79
.db 80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95
.db 96,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111
.db 112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127
.db 128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143
.db 144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159
.db 160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175
.db 176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191
.db 192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207
.db 208,209,210,211,212,213,214,215,216,217,218,219,220,221,222,223
.db 224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239
.db 240,241,242,243,244,245,246,247,248,249,250,251,252,253,254,255

.db 0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15
.db 16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31
.db 32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47
.db 48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63
.db 64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79
.db 80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95
.db 96,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111
.db 112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127
.db 128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143
.db 144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159
.db 160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175
.db 176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191
.db 192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207
.db 208,209,210,211,212,213,214,215,216,217,218,219,220,221,222,223
.db 224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239
.db 240,241,242,243,244,245,246,247,248,249,250,251,252,253,254,254
.db 255,254,253,252,251,250,249,248,247,246,245,244,243,242,241,240
.db 239,238,237,236,235,234,233,232,231,230,229,228,227,226,225,224
.db 223,222,221,220,219,218,217,216,215,214,213,212,211,210,209,208
.db 207,206,205,204,203,202,201,200,199,198,197,196,195,194,193,192
.db 191,190,189,188,187,186,185,184,183,182,181,180,179,178,177,176
.db 175,174,173,172,171,170,169,168,167,166,165,164,163,162,161,160
.db 159,158,157,156,155,154,153,152,151,150,149,148,147,146,145,144
.db 143,142,141,140,139,138,137,136,135,134,133,132,131,130,129,128
.db 127,126,125,124,123,122,121,120,119,118,117,116,115,114,113,112
.db 111,110,109,108,107,106,105,104,103,102,101,100,99,98,97,96
.db 95,94,93,92,91,90,89,88,87,86,85,84,83,82,81,80
.db 79,78,77,76,75,74,73,72,71,70,69,68,67,66,65,64
.db 63,62,61,60,59,58,57,56,55,54,53,52,51,50,49,48
.db 47,46,45,44,43,42,41,40,39,38,37,36,35,34,33,32
.db 31,30,29,28,27,26,25,24,23,22,21,20,19,18,17,16
.db 15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0

.db 255,254,253,252,251,250,249,248,247,246,245,244,243,242,241,240
.db 239,238,237,236,235,234,233,232,231,230,229,228,227,226,225,224
.db 223,222,221,220,219,218,217,216,215,214,213,212,211,210,209,208
.db 207,206,205,204,203,202,201,200,199,198,197,196,195,194,193,192
.db 191,190,189,188,187,186,185,184,183,182,181,180,179,178,177,176
.db 175,174,173,172,171,170,169,168,167,166,165,164,163,162,161,160
.db 159,158,157,156,155,154,153,152,151,150,149,148,147,146,145,144
.db 143,142,141,140,139,138,137,136,135,134,133,132,131,130,129,128
.db 127,126,125,124,123,122,121,120,119,118,117,116,115,114,113,112
.db 111,110,109,108,107,106,105,104,103,102,101,100,99,98,97,96
.db 95,94,93,92,91,90,89,88,87,86,85,84,83,82,81,80
.db 79,78,77,76,75,74,73,72,71,70,69,68,67,66,65,64
.db 63,62,61,60,59,58,57,56,55,54,53,52,51,50,49,48
.db 47,46,45,44,43,42,41,40,39,38,37,36,35,34,33,32
.db 31,30,29,28,27,26,25,24,23,22,21,20,19,18,17,16
.db 15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0
.db 0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15
.db 16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31
.db 32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47
.db 48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63
.db 64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79
.db 80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95
.db 96,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111
.db 112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127
.db 128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143
.db 144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159
.db 160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175
.db 176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191
.db 192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207
.db 208,209,210,211,212,213,214,215,216,217,218,219,220,221,222,223
.db 224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239
.db 240,241,242,243,244,245,246,247,248,249,250,251,252,253,254,255










